
%macro csbmva_feed_pd_repool(model_key    =  ,time_key        =  ,
                      		 input_ds_lib =  ,input_ds_nm     =  ,
                      		 out_feed_lib =  ,feed_ds_nm_pool =  ,
							 quantile_pd  =  ,quantile_lgd 	  =  ,
                             quantile_ccf =  ,quantile_scr    =  ,
                             interval_pd  =  ,interval_lgd 	  =  ,
                             interval_ccf =  ,interval_scr    =  ,
                      		 action       =  ,model_type   =  );

*==================================================================;
* declaring local macro variables ;
*==================================================================;

%local period_end_dttm  m_level_sk scheme_type mm_ext_key range_table;

data _null_;
  a = put(&time_key,cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);

*==================================================================;
* capturing few values from source into macro variables ;
*==================================================================;

proc sql noprint;
  select  level_sk
    into  :m_level_sk
  from &input_ds_lib..&input_ds_nm;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Subsetting data if model type is PDA and getting ;
* few values from ANALYTICAL_MODEL_DIM ;
*==================================================================;

proc sql;
  create table feed_input_prep as
    select a.*, 
      c.offset_rt as offset, c.factor, c.cut_off_score 
	from &input_ds_lib..&input_ds_nm as a
  	     %if &model_type=%str(&PD_APP_MODEL) %then %do;
           inner join dim.application_dim(keep = application_rk outcome_cd
										  where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT )) as b 
		   on (a.record_rk = b.application_rk)
		 %end;
            inner join 
         dim.analytical_model_dim(where = (valid_start_dttm <= &period_end_dttm
                                                        <= valid_end_dttm )) as c
    on (a.model_rk = c.model_rk)
	;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* generating range table ;
*==================================================================;

%let range_table = _&model_key._repool_range;

%if %kupcase(&action) = Q %then %do;
  %csbmva_quantile_repool(model_key     = &model_key   ,time_key     = &time_key      ,
                          input_ds_lib  = work		   ,input_ds_nm  = feed_input_prep,
						  output_ds_lib = csbscr	   ,output_ds_nm = &range_table   ,
						  quantile_pd   = &quantile_pd ,quantile_lgd = &quantile_lgd  ,
                          quantile_ccf  = &quantile_ccf,quantile_scr = &quantile_scr);
%end;
%else %if %kupcase(&action) = I %then %do;
  %csbmva_interval_repool(model_key     = &model_key   ,time_key     = &time_key      ,
                          input_ds_lib  = work		   ,input_ds_nm  = feed_input_prep,
						  output_ds_lib = csbscr	   ,output_ds_nm = &range_table   ,
						  interval_pd   = &interval_pd ,interval_lgd = &interval_lgd  ,
                          interval_ccf  = &interval_ccf,interval_scr = &interval_scr);
%end;

*==================================================================;
* finding the pool_schem_type of the model ;
*==================================================================;

proc sql noprint;
  select kupcase(b.pool_scheme_type_cd)
    into :scheme_type
  from dim.analytical_model_dim a,&csb_apdm_libref..pool_scheme b
  where a.pool_scheme_sk = b.pool_scheme_sk
  and   a.model_rk = &model_key
  and   a.valid_start_dttm <= &period_end_dttm <= a.valid_end_dttm;
quit;
%bankfdn_rcset(&sqlrc);

%let scheme_type = &scheme_type;
%let scheme_type = %str(%')&scheme_type%str(%');

*==================================================================;
* Finding the model_key value for name of model_dev_fact ;
*==================================================================;

proc sql noprint;
	select source_model_master_key into :mm_ext_key
	from dim.analytical_model_dim
	where model_rk = &model_key;
quit;
%let mm_ext_key = &mm_ext_key;

*==================================================================;
* generating Pool level feed ;
*==================================================================;

/* START - Calculating Actual data */

/* Assigning Pool */
proc sql;
  create table feed_repool_temp1 as 
    select a.*,b.pool_name,b.pool_seq_no
	from feed_input_prep as a
           right outer join
         csbscr.&range_table as b
	on ( b.from_score < a.score_points <= b.to_score )
	where pool_seq_no ne .;
quit;
%bankfdn_rcset(&sqlrc);

/* Grouping the actual data at POOL */
proc sql;
  create table feed_repool_temp2 as
    select distinct pool_name, pool_seq_no,
      sum(case when record_rk ne . then 1 else 0 end) as no_of_records_actual,
	  coalesce(sum(actual_good_bad),0) as no_of_actual_bads,
	  &model_key as model_rk,
	  &time_key as score_time_sk,
      &m_level_sk as level_sk
	from feed_repool_temp1 
  	group by pool_seq_no;
quit;
%bankfdn_rcset(&sqlrc);

/* Calculating no_of_records_actual_all */
%if &model_type= &PD_APP_MODEL  %then %do;
	proc sql;
	  create table feed_repool_act_all as 
	    select
			pool_seq_no,
			sum(case when record_rk ne . then 1 else 0 end) as no_of_records_actual_all
		from &input_ds_lib..&input_ds_nm a
	           right outer join
	         csbscr.&range_table as b
		on ( b.from_score < a.score_points <= b.to_score )
		group by pool_seq_no;
	quit;
%end;

/* END - Calculating Actual data */

/* START - Calculating DEV data */
proc sql;
  create table feed_repool_temp1_dev as 
    select  a.*,
			b.pool_name, b.pool_seq_no
	from &cs_fact_lib.._&mm_ext_key._dev_fact as a
           right outer join
         csbscr.&range_table as b
	on ( b.from_score < a.score_points <= b.to_score )
    where pool_seq_no ne .;
quit;
%bankfdn_rcset(&sqlrc);

/* Grouping the dev data at POOL */
proc sql;
  create table feed_repool_temp2_dev as
    select pool_name, pool_seq_no ,
           sum(case when record_rk ne . then 1 else 0 end) as no_of_records_dev,
		   max(score_points) as max_score_points,
	  	   avg(estimated_pd) as estimated_pd
    from feed_repool_temp1_dev
	group by pool_name,pool_seq_no;
quit;
%bankfdn_rcset(&sqlrc);

/* END - Calculating DEV data */

*==================================================================;
* Create o/p dataset ;
*==================================================================;

proc sql;

	CREATE TABLE &out_feed_lib..&feed_ds_nm_pool
	(
	POOL_SEQ_NO NUMERIC(8) ,
	POOL_NAME CHARACTER(300) FORMAT = $300. INFORMAT = $300.  ,
	NO_OF_RECORDS_DEV NUMERIC(8)  ,
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		NO_OF_RECORDS_ACTUAL_ALL NUMERIC(8)  ,
	%end;
	NO_OF_RECORDS_ACTUAL NUMERIC(8)  ,
	NO_OF_ACTUAL_BADS NUMERIC(8)  ,
	MAX_SCORE_POINTS NUMERIC(8) ,
	ESTIMATED_PD  NUMERIC(8) ,
	MODEL_RK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	LEVEL_SK NUMERIC(8) NOT NULL ,
    SCORE_TIME_SK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL 
	);

    create unique index xpk_feed_pd on &out_feed_lib..&feed_ds_nm_pool(model_rk,score_time_sk,pool_seq_no);

quit;

proc sql;
  insert into &out_feed_lib..&feed_ds_nm_pool
  	(
		POOL_SEQ_NO ,
		POOL_NAME ,
		NO_OF_RECORDS_ACTUAL ,
		NO_OF_ACTUAL_BADS ,
		NO_OF_RECORDS_DEV ,
		MAX_SCORE_POINTS ,
		ESTIMATED_PD ,
		MODEL_RK ,
		SCORE_TIME_SK,
		LEVEL_SK
	    %if &model_type=%str(&PD_APP_MODEL) %then %do;
			,NO_OF_RECORDS_ACTUAL_ALL
		%end;
	)
    select 
		a.pool_seq_no,
		a.pool_name,
		a.no_of_records_actual,
		a.no_of_actual_bads,
		case when b.no_of_records_dev = . then 0 
			else b.no_of_records_dev
        end as no_of_records_dev,
		b.max_score_points,
		b.estimated_pd,
	    &model_key as model_rk,
	    &time_key as score_time_sk,
        &m_level_sk as level_sk
		%if &model_type= &PD_APP_MODEL  %then %do;
			,y.no_of_records_actual_all
		%end;
    from
		feed_repool_temp2 as a, 
		feed_repool_temp2_dev as b
		%if &model_type= &PD_APP_MODEL  %then %do; 
			 ,feed_repool_act_all y
		%end;
	where a.pool_seq_no = b.pool_seq_no
	%if &model_type= &PD_APP_MODEL  %then %do;
    	and b.pool_seq_no = y.pool_seq_no 
	%end; 
    ;  
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_feed_pd_repool;
