/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_ui_get_feed_mm.sas
Function       : This macro creates the output dataset for displaying feed for 
				 MM reports

Authors        :  CSB Team
SAS            :  9.3

Called-by      :  Mid Tier
Calls          :  None

Input Datasets :  _<MODEL_RK>_<SCORE_TIME_SK>_PD_POOL_FD
								OR
                  _<MODEL_RK>_<SCORE_TIME_SK>_LGD_POOL_FD
                                OR
				  _<MODEL_RK>_<SCORE_TIME_SK>_CCF_POOL_FD
                   
Output Datasets:  Dataset for displaying feed

Logic          :  creates the output dataset displaying feed

Parameters     :  model_key  	  	 -> Key of the model
                  score_time_sk      -> Score time point Key
                  measure_cd    	 -> Code for the measure
                  isFilterApplied    -> Specify whether filter is appliad or not.
                  isRepooled    	 -> Specify whether repooled  or not.
                  isBackTestedReport -> Specify whether backtesting reports or not.
				  output_ds			 -> Output dataset

**********************************************************************************************/

%macro csbmva_ui_get_feed_mm(model_key  = ,score_time_sk = ,
							 measure_cd = ,isFilterApplied = ,
							 isRepooled = ,isBackTestedReport = ,
							 output_ds  = );

%let model_key = &model_key;
%let score_time_sk = &score_time_sk;
%let measure_cd = %lowcase(&measure_cd);

*==================================================;
* Defining local macro variables ;
*==================================================;

%local  feed_lib model_type feed_ds_suffix m_feed_nm
		trans_model_key;

*==================================================;
* Setting up source library ;
*==================================================;

%if &isFilterApplied = true or &isRepooled = true %then %do;
	%let feed_lib = csbscr;
%end;
%else %do;
	%let feed_lib = csbfact;
%end;

*==================================================;
* Find model type code ;
*==================================================;

proc sql noprint;
  select model_type_cd into :model_type
  from &csb_apdm_libref..model_master_extension
  where model_key = &model_key;
quit;
%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

*==================================================;
* Changing the model_key ;
*==================================================;

%if &isBackTestedReport = true %then %do;
	%let trans_model_key = &model_key;
%end;
%else %do;
	proc sql noprint;
	  select b.model_rk into :trans_model_key
	  from &csb_apdm_libref..model_master_extension a, dim.analytical_model_dim b
	  where a.model_key = b.source_model_master_key
	  and a.model_key = &model_key;
	quit;
	%let trans_model_key = &trans_model_key;
%end;

*==================================================;
* Generating feed dataset name ;
*==================================================;

%if &measure_cd = nmltst %then %do;
	%let feed_lib = work;
	%let m_feed_nm = normal_feed;
	%goto outptut;
%end;

%if &isBackTestedReport = true %then %do;
	%let feed_ds_suffix = _oot;
%end;

%if %kupcase(&model_type) eq %kupcase(&PD_APP_MODEL)
				or 
	%kupcase(&model_type) eq %kupcase(&PD_ONG_MODEL)
%then %do;
	%let m_feed_nm = _&trans_model_key._&score_time_sk.&feed_ds_suffix._pd_pool_fd;
%end;
%else %if %kupcase(&model_type) eq %kupcase(&LGD_MODEL)
%then %do;
	%let m_feed_nm = _&trans_model_key._&score_time_sk.&feed_ds_suffix._lgd_pool_fd;
%end;
%else %if %kupcase(&model_type) eq %kupcase(&CCF_MODEL)
%then %do;
	%let m_feed_nm = _&trans_model_key._&score_time_sk.&feed_ds_suffix._ccf_pool_fd;
%end;

*==================================================;
* Creating output dataset ;
*==================================================;

%outptut:

/*Start: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for score shift index calculation */
%local input_col_act_tmp input_col_dev_tmp cnt_stab_col;

%let cnt_stab_col=0;
proc sql noprint;
select count(*) into :cnt_stab_col from dictionary.columns 
where strip(kupcase(libname)) || "." || strip(kupcase(memname))=%kupcase("&feed_lib..&m_feed_nm") and kupcase(name)='NO_OF_RECORDS_ACTUAL_ALL';
quit;
%let cnt_stab_col=&cnt_stab_col;

%if &cnt_stab_col gt 0 %then %do; /*Additional condition for hot_fix related to score shift index calculation*/
	data &output_ds;
	set &feed_lib..&m_feed_nm 
	%if "&measure_cd" eq "sysstbindx" %then %do;
		(drop= NO_OF_RECORDS_ACTUAL )
	%end;
	%else %do;
		(drop= NO_OF_RECORDS_ACTUAL_ALL )
	%end;
	;
	%if "&measure_cd" eq "sysstbindx" %then %do;
		rename NO_OF_RECORDS_ACTUAL_ALL=NO_OF_RECORDS_ACTUAL;
	%end;
	run;
   
%end;
%else %do;
	data &output_ds;
	set &feed_lib..&m_feed_nm;
	run;
	
%end;

/*End: Modified for hotfix defect id:S1106744 to consider all records, accepted and rejected for SSI calculation */

%mend csbmva_ui_get_feed_mm;
