
	/* This macro builds the back testing abt and returns the status in the macro 
		m_abt_bld_status as 0 for success and 1 for failure */
%macro dabt_build_bck_plg_abt_wrapper(m_model_id=, m_pool_scheme_no =,m_mdl_type=, m_abt_bld_dt=,m_abt_stk_flg=, m_abt_stk_size=, m_replace_abt_flg=, 
									m_perform_scoring_flg =,m_abt_libref=,
									m_abt_tbl_nm=, m_scored_ds_nm=, m_scored_libref=,
									m_abt_lst_prc_by_usr=, m_abt_build_status=) / des='dabt_build_bck_plg_abt_wrapper';

	%let m_model_id = &m_model_id;
	%let m_abt_libref = &m_abt_libref;
	%let m_abt_tbl_nm = &m_abt_tbl_nm;
	%let m_mdl_type = &m_mdl_type;
	%let m_mdl_type_low = %lowcase(&m_mdl_type);
	%let m_perform_scoring_flg = %upcase(&m_perform_scoring_flg);
	%let m_replace_abt_flg = %upcase(&m_replace_abt_flg);

	%if %symexist(job_rc)=0 %then %do;
		%global job_rc;
	%end;
	%if %symexist(sqlrc)=0 %then %do;
		%global sqlrc;
	%end;
	
	%let job_rc = 0;
	
	%let m_src_lib = ;
	/* assigns the ADM library */
	%dabt_assign_libs(m_project_sk=., src_lib=m_src_lib, m_workspace_type=DUMMY);
						
	%let m_project_sk = ;
	%let m_model_sk = ;
	%let m_level_sk = ;
	proc sql noprint;
		select project_sk, model_sk, level_sk into :m_project_sk, :m_model_sk, :m_level_sk
		from &m_src_lib..model_master
		where model_id = "&m_model_id.";	
	quit;
	%dabt_err_chk(type=SQL);
	%let m_project_sk = &m_project_sk;
	%let m_model_sk = &m_model_sk;
	%let m_level_sk = &m_level_sk;
						 
		
	%if &m_mdl_type = BCK %then %do;
		%let m_scr_libref = &DABT_BACKTESTING_SCR_LIBREF.;
	%end;
	%else %do;
		%let m_scr_libref = &DABT_POOLING_SCR_LIBREF.;
	%end;
	%if &m_abt_libref eq %then %do;
		%if &m_mdl_type = BCK %then %do;
			%let m_abt_libref = &DABT_BACKTESTING_ABT_LIBREF.;
		%end;
		%else %do;
			%let m_abt_libref = &DABT_POOLING_ABT_LIBREF.;
		%end;
	%end;

	%if &m_abt_tbl_nm eq %then %do;
		%put ERROR: value not assigned to the parameter m_abt_tbl_nm. Pass the name with which the abt is to be created.;
		%return;
	%end;
	%if &m_perform_scoring_flg eq Y %then %do;
		%if &m_scored_ds_nm eq %then %do;
			%put ERROR: value not assigned to the parameter m_scored_ds_nm. Pass the name with which the scored dataset is to be created.;
			%return;
		%end;
		%if &m_scored_libref eq %then %do;
			%let m_scored_libref = &m_abt_libref;
		%end;
	%end;

	%if %sysfunc(exist(&m_abt_libref..&m_abt_tbl_nm)) %then %do;
		%if &m_replace_abt_flg eq Y %then %do;
			proc sql noprint;
				drop table &m_abt_libref..&m_abt_tbl_nm;
			quit;
			%dabt_err_chk(type=SQL);
		%end;
		%else %do;
			%put ERROR: &m_abt_tbl_nm. already exists in the library &m_abt_libref.;
			%return;
		%end;
	%end;
	%if &m_perform_scoring_flg eq Y %then %do;
		%if %sysfunc(exist(&m_scored_libref..&m_scored_ds_nm)) %then %do;
			%if &m_replace_abt_flg eq Y %then %do;
				proc sql noprint;
					drop table &m_scored_libref..&m_scored_ds_nm;
				quit;
				%dabt_err_chk(type=SQL);
			%end;
			%else %do;
				%put ERROR: &m_scored_ds_nm. already exists in the library &m_scored_libref.;
				%return;
			%end;
		%end;
	%end;

	%if &m_mdl_type = BCK %then %do;
		/* The UI for backtesting doesn't update this table, hence it is updated here */
		%let m_lst_prc_dt = %sysfunc(datetime());
		proc sql noprint ;
			update &m_src_lib..model_master
			set last_processed_dttm = &m_lst_prc_dt,
				last_processed_by_user = "&m_abt_lst_prc_by_usr",
				last_evaluation_run_dttm = &m_lst_prc_dt,
	/*			last_evaluation_stacked_type_flg = "&m_abt_stk_flg",*/
				last_evaluation_status_sk = (select status_sk from &m_src_lib..status_master
								where kupcase(strip(status_cd)) = "RUNNING")
			where model_sk = &m_model_sk.;
		quit;
		%dabt_err_chk(type=SQL);
	%end;

	%let m_abt_sts = 0;
	%if &job_rc le 4 %then %do;
		%dabt_build_abt(m_project_sk=&m_project_sk, m_model_sk=&m_model_sk, m_abt_bld_dt=&m_abt_bld_dt, m_abt_stk_flg=&m_abt_stk_flg, m_abt_stk_size=&m_abt_stk_size,
										m_abt_type=&m_mdl_type, m_abt_libref=&m_abt_libref, m_scr_libref=&m_scr_libref,
										m_abt_tbl_nm=&m_abt_tbl_nm, m_abt_lst_prc_by_usr=&m_abt_lst_prc_by_usr, m_abt_bld_sts=m_abt_sts);
										
		proc printto log = "&m_log_path.build_&m_mdl_type_low._wrapper.log"  &log_clear;
		run;
		%dabt_err_chk(type=DATA);
	%end;

	%let app_scr_sts = 0;
	%if &job_rc le 4 and &m_abt_sts eq 0 %then %do;
		%if &m_perform_scoring_flg eq Y %then %do;
			%let m_scr_path = ;
			%let m_tmp_lib = ;
			%let m_src_lib = ;
			%let m_scoring_code_path = ;
			/* assigns the temporary library and ADM library */
			%dabt_assign_libs(m_project_sk=&m_project_sk, src_lib=m_src_lib,tmp_lib=m_tmp_lib,m_workspace_type=&m_mdl_type, m_called_for_abt_processing_flg=N,
							 m_abt_or_template_sk=&m_model_sk., scr_path=m_scr_path, scoring_code_path=m_scoring_code_path);
			%dabt_err_chk(type=SQL);
			%dabt_err_chk(type=DATA);
			%dabt_err_chk(type=LIB);

			%let app_scr_sts = ;
			%dabt_apply_scoring_code(m_scoring_abt_table_nm=&m_abt_libref..&m_abt_tbl_nm,
									m_output_table_nm=&m_scored_libref..&m_scored_ds_nm,
									m_level_sk=&m_level_sk,m_port_path=&m_scoring_code_path, m_scr_status=app_scr_sts,
									m_scoring_code_file_nm=scoring_code_&m_model_id..sas);
									
			proc printto log = "&m_log_path.build_&m_mdl_type_low._wrapper.log";
			run;
			%dabt_err_chk(type=DATA);
		%end;
	%end;

	%if &job_rc le 4 and &m_abt_sts eq 0 and &app_scr_sts eq 0 %then %do;
		/* return the status code 0 for success */
		%let &m_abt_build_status = 0;
	%end;
	%else %do;
		/* return the status code 0 for failure */
		%let &m_abt_build_status = 1;
	%end;

	%if &m_mdl_type = BCK %then %do;
		%let m_lst_prc_dt = %sysfunc(datetime());
		proc sql noprint ;
			update &m_src_lib..model_master
			set last_processed_dttm = &m_lst_prc_dt,
				last_evaluation_status_sk = (select status_sk from &m_src_lib..status_master
										where kupcase(strip(status_cd)) = 
									%if &&&m_abt_build_status eq 0 %then %do;
										"SUCCESS"
									%end;
									%else %do;
										"FAIL"
									%end;
										)
			where model_sk = &m_model_sk.;
		quit;
		%dabt_err_chk(type=SQL);
	%end;

	%if &job_rc le 4 %then %do;
		%let m_purpose_sk = .;
		proc sql noprint;
			select purpose_sk into :m_purpose_sk
			from &m_src_lib..project_master
			where project_sk = &m_project_sk.;
		quit;
		%dabt_err_chk(type=SQL);
		%let m_purpose_sk = &m_purpose_sk.;

		/* This macro variable will be assigned the value N, if the post action macro executes without error,
		 else will have value Y. */
		%let post_action_err_flg = ;
		/* Modified for pooling added as part of hotfix 2014: S1109569 */
		%if &m_mdl_type = BCK %then %do;
			%dabt_call_post_action_macro(m_action_cd=BCK_JOB, m_purpose_sk=&m_purpose_sk,
											m_model_sk=&m_model_sk, m_post_action_flg=post_action_err_flg);
		%end;
		%else %do;
			%dabt_call_post_action_macro(m_action_cd=PLG_JOB, m_purpose_sk=&m_purpose_sk,
								m_model_sk=&m_model_sk, m_pool_scheme_no=&m_pool_scheme_no, m_post_action_flg=post_action_err_flg);
		%end;
		%dabt_err_chk(type=DATA);
		%dabt_err_chk(type=SQL);
		%dabt_err_chk(type=LIB);
		%if &post_action_err_flg = Y %then %do;
			%let job_rc = 5;
		%end;
	%end;
	
	proc printto log=log;
	quit;

%mend dabt_build_bck_plg_abt_wrapper;
