%macro csbmva_cal_measures_ccf(model_key      = ,time_key       = ,
                               model_type     = ,feed_lib       = ,
					           feed_ds_pool   = ,feed_ds_record = ,
					           measure_ds_lib = ,measure_ds_nm  = );

%local 	temp_model_type measure_sk_lst
        measure_cd_lst  measure_cnt
		default_code    m_measure_sk
        m_measure_cd    m_measure_val
		i
		;

proc sql;
  create table &measure_ds_lib..&measure_ds_nm(
    MODEL_RK num(8),
	SCORE_TIME_SK num(8),
    MEASURE_SK num(8),
    MEASURE_VALUE num(8)
  );
quit;

%if &model_type = &PD_ONG_MODEL %then %do;
  %let temp_model_type  = 'PDO';
%end;
%else %if &model_type = &PD_APP_MODEL %then %do;
  %let temp_model_type  = 'PDA';
%end;
%else %if &model_type = &LGD_MODEL %then %do;
  %let temp_model_type  = 'LGD';
%end;
%else %if &model_type = &CCF_MODEL %then %do;
  %let temp_model_type  = 'CCF';
%end;

%bankfdn_rcset(&syserr);

proc sql noprint;

  select a.measure_sk , a.measure_cd , count(*)  
    into :measure_sk_lst separated by '#',
         :measure_cd_lst separated by '#',
		 :measure_cnt
  from &cs_dim_lib..measure_dim as a, &cs_bridge..model_type_x_measure as b
  where a.measure_sk = b.measure_sk 
  and Kupcase(b.model_type_cd) = %kupcase(&temp_model_type);

quit;
%bankfdn_rcset(&sqlrc);

proc sql noprint;
  create table other_msr as
  select a.measure_cd
  from &cs_dim_lib..measure_dim as a, &cs_bridge..model_type_x_measure as b
  where a.measure_sk = b.measure_sk 
  and kupcase(measure_level_cd) = 'OTHE'
  and Kupcase(b.model_type_cd) = %kupcase(&temp_model_type);

quit;
%bankfdn_rcset(&sqlrc);

data _null_;
  set other_msr;
  call symput(measure_cd,'.');
run;

%let default_code=;
%csbmva_alldefaults_nodefaults(&feed_lib..&feed_ds_pool,
                               no_of_records_actual,
                               No_Of_Actual_Bads,
                               default_code);

%if &default_code ne 'NONE' %then %do;

  %csbmva_sysstbindx(input_ds = &feed_lib..&feed_ds_pool,output_ds = ssi,flag = 2);
  proc sql noprint;
    select sum(sysstbindx) into :sysstbindx 
	from work.ssi;
  quit;

  %do i = 1 %to &measure_cnt;
    %let m_measure_sk = %kscan(&measure_sk_lst,&i,%str(#));
	%let m_measure_cd = %kscan(&measure_cd_lst,&i,%str(#));
    %let m_measure_val = .;

	%if %kupcase(&m_measure_cd) = SYSSTBINDX %then %do;
      %let m_measure_val = &sysstbindx;
	%end;

    proc sql;
	  insert into &measure_ds_lib..&measure_ds_nm
	    (model_rk, score_time_sk, measure_sk, measure_value)
		  values
		(&model_key, &time_key, &m_measure_sk, &m_measure_val);
	quit;
	%bankfdn_rcset(&sqlrc);
  %end;

%end;

%else %do;

  %csbmva_sysstbindx(input_ds = &feed_lib..&feed_ds_pool,output_ds = ssi,flag=2);
  proc sql noprint;
    select sum(sysstbindx) into :sysstbindx 
	from work.ssi;
  quit;
/**/
/*  %csbmva_ar(input_ds = &feed_lib..&feed_ds_pool,output_ds=ar_data,flag=2,model_type =&model_type);*/
/*   proc sql noprint;*/
/*    select ar into :ar*/
/*	from work.ar_data;*/
/*  quit;*/

  %csbmva_errormeasures(input_ds = &feed_lib..&feed_ds_pool,input_ds_estvar = Estimated_CCF,input_ds_actvar = Actual_CCF,output_ds = work.mse_mad);
  proc sql noprint;
    select mse, mad into :mse, :mad
	from work.mse_mad;
  quit;

  %csbmva_mape(input_ds=&feed_lib..&feed_ds_pool,output_ds=mape1,flag=2,model_type =&model_type);
   proc sql noprint;
    select mape into :mape
	from work.mape1;
  quit;

  %csbmva_chsqp(input_ds = &feed_lib..&feed_ds_pool,output_ds = WORK.chsqp,flag=2,model_type =&model_type);
  proc sql noprint;
    select chsqp into :chsqp
	from work.chsqp;
  quit;

  %csbmva_corr(input_ds = &feed_lib..&feed_ds_pool,output_ds = WORK.corr,flag=2,model_type =&model_type);
  data _null_;
    set corr;
	call symput(_type_,measure_value);
  run;

    %do i = 1 %to &measure_cnt;
    %let m_measure_sk = %kscan(&measure_sk_lst,&i,%str(#));
	%let m_measure_cd = %kscan(&measure_cd_lst,&i,%str(#));
	%let m_measure_val = .;
    %let m_measure_val = &&&m_measure_cd;

    proc sql;
	  insert into &measure_ds_lib..&measure_ds_nm
	    (model_rk, score_time_sk, measure_sk, measure_value)
		  values
		(&model_key, &time_key, &m_measure_sk, &m_measure_val);
	quit;
	%bankfdn_rcset(&sqlrc);
  %end;

%end;

%mend csbmva_cal_measures_ccf;
