/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_feed_pd_mip.sas
Function       : This macro creates the feed table for PD Model for Model Input Reports.

Authors        :  CSB Team
Date           :  29Oct2010
SAS            :  9.2

Called-by      :   
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  SCORE_CARD_DIM (CSBDIM library)
                  SCORE_CARD_VARIABLE_DIM (CSBDIM library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_PD_FACT (CSB FACT library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_SC_FACT (CSB FACT library)
                  _<MODEL_RK>_SC_RANGE_FACT (CSB FACT library)
                  
Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_PD_ATTR_FD_MIP (CSB FACT library)
                  _<MODEL_RK>_<SCORE_TIME_SK>_PD_ATTR_FD_MIP (CSB FACT library)

Logic          :  

Parameters     :  MODEL_KEY         -> Model_RK of the model.
                  TIME_KEY          -> Score_time_sk of the model.
                  OUT_LIB_NM        -> Library where the feed dataset will be created.
                  OUT_LIB_NM_DIM    -> Library where Dimensions are present.
                  OUT_LIB_NM_FACT   -> Library where Facts are present.
                  OUT_TABLE_NM_POOL -> Name of the attribute level feed table.
                  OUT_TABLE_NM_ACCT -> Name of the score range level feed table.

**********************************************************************************************/
%macro csbmva_feed_pd_mip(model_key         = ,time_key          = ,
                          out_lib_nm        = ,out_lib_nm_dim    = ,
                          out_lib_nm_fact   = ,out_table_nm_pool = , 
	                      out_table_nm_acct = ,isDevelopmentCycle = );

%local modelsk levelsk model_type;

*==================================================================;
* Finding the model_key from model_master_extension ;
*==================================================================;

%if &isDevelopmentCycle = true %then %do;
	%let mm_ext_key = &model_key;
%end;
%else %do;
	proc sql noprint;
		select source_model_master_key into :mm_ext_key
		from dim.analytical_model_dim
		where model_rk = &model_key;
	quit;
	%let mm_ext_key = &mm_ext_key;
%end;

proc sql noprint;
	select model_type_cd into :model_type
	from &csb_apdm_libref..MODEL_MASTER_EXTENSION b
	where model_key = &mm_ext_key
	;
quit;

%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

*==============================================================;
* Creating Attribute level feed ;
*==============================================================;

/* To populate developemnt data for all respective deminsions*/
proc sql;
  create table &debug_lib..all_dev_attribute_levels as
    select  
      a.model_rk as model_master_rk,
      a.variable_sk,
	  a.variable_group_label as attr_name length = 200,
      a.variable_group_no as seq_no , 
      b.variable_name as var_name length = 200,
      max(a.score_card_points) as scorecard_points, 
      max(a.no_of_records) as no_of_records_dev, 
      max(a.no_of_defaults) as no_of_defaults_dev, 
      max(a.average_score) as avg_score_points
    from &out_lib_nm_dim..score_card_dim as a,
           &out_lib_nm_dim..score_card_variable_dim as b
    where a.variable_sk = b.variable_sk
	and a.variable_group_no ne -2
    and a.model_rk = &mm_ext_key
    group by a.model_rk,a.variable_sk,var_name,attr_name, seq_no;
quit;
%bankfdn_rcset(&sqlrc);

/* To populate actual data for all respective deminsions */

proc sql;
  create table &debug_lib..&out_table_nm_pool._sql as
    select 
	  a.model_rk,
 	  a.record_rk,
	  a.predicted_good_bad as predicted_good_bad,
      a.actual_good_bad as actual_good_bad,
      a.score_time_sk,
	  a.level_sk,
      c.variable_sk,
	  c.variable_group_label as attr_name length = 200, 
	  c.variable_group_no as seq_no,
      d.variable_name as var_name length = 200
   from &out_lib_nm_fact.._&model_key._&time_key._pd_fact as a
                 inner join
       &out_lib_nm_fact.._&model_key._&time_key._sc_fact as b
   on (a.record_rk =  b.record_rk)
	             inner join
       &out_lib_nm_dim..score_card_dim as c
   on ( b.score_card_sk = c.score_card_sk) 
                 inner join
       &out_lib_nm_dim..score_card_variable_dim as d
   on (c.variable_sk = d.variable_sk)
  ;

quit;
%bankfdn_rcset(&sqlrc);

/* grouping at attribute level */
%if &model_type=%str(&PD_APP_MODEL) %then %do;
	proc sql;
	  create table &debug_lib..&out_table_nm_pool._app as
	    select model_rk,
	      score_time_sk,
	      level_sk,
	      variable_sk,
	      var_name, 
	      attr_name, 
		  seq_no,
		  count(a.record_rk) as no_of_records_actual_all
	  from &debug_lib..&out_table_nm_pool._sql a
	  group by model_rk,score_time_sk,level_sk,variable_sk,var_name,attr_name,seq_no;
	quit;
%end;
%bankfdn_rcset(&sqlrc);

/* grouping at attribute level */
proc sql;
  create table &debug_lib..&out_table_nm_pool._tmp as
    select a.model_rk,
      a.score_time_sk,
      a.level_sk,
      a.variable_sk,
      a.var_name, 
      a.attr_name, 
	  a.seq_no,
	  count(a.record_rk) as no_of_records_actual,
	  sum(a.actual_good_bad) as no_of_defaults_actual
  from &debug_lib..&out_table_nm_pool._sql a
  %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
  		on (a.record_rk = b.application_rk)

  %end;
  group by model_rk,score_time_sk,level_sk,variable_sk,var_name,attr_name,seq_no;
quit;
%bankfdn_rcset(&sqlrc);
%if &model_type=%str(&PD_APP_MODEL) %then %do;
	%let alias = b;
%end;
%else %do;
	%let alias = a;
%end;
proc sql;
  create table &debug_lib..&out_table_nm_pool. as
  	select
      &alias..model_rk,
      &alias..score_time_sk,
      &alias..level_sk,
      &alias..variable_sk,
      &alias..var_name, 
      &alias..attr_name, 
	  &alias..seq_no,
	  a.no_of_records_actual,
	  a.no_of_defaults_actual 
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			, b.no_of_records_actual_all
		%end;
		from &debug_lib..&out_table_nm_pool._tmp a 
			%if &model_type=%str(&PD_APP_MODEL) %then %do;
				right join &debug_lib..&out_table_nm_pool._app b
				on (a.model_rk = b.model_rk and
					a.score_time_sk = b.score_time_sk and
					a.level_sk = b.level_sk and
					a.variable_sk = b.variable_sk and
					a.var_name = b.var_name and
					a.attr_name = b.attr_name and
					a.seq_no = b.seq_no)
			%end;
		;
quit;
%bankfdn_rcset(&sqlrc);

/* Merge development data with the actual data for all respective deminsions */
proc sort data  = &debug_lib..all_dev_attribute_levels;
  by variable_sk seq_no;
run;

proc sort data  = &debug_lib..&out_table_nm_pool.;
  by variable_sk seq_no;
run;

data  &debug_lib..&out_table_nm_pool.;
  merge &debug_lib..all_dev_attribute_levels (in=a ) &debug_lib..&out_table_nm_pool. (in = b);
  by  variable_sk seq_no;
  if a =1 and  b =1 then output &debug_lib..&out_table_nm_pool.;
  if a =1 and b =0 then output &debug_lib..&out_table_nm_pool.;
  if a =0 and b = 1 then output &debug_lib..&out_table_nm_pool.;
run;
%bankfdn_rcset(&syserr);

proc sql noprint;
  select max(level_sk) into
    :levelsk
  from &debug_lib..&out_table_nm_pool.;
quit;
%bankfdn_rcset(&sqlrc);

data  &out_lib_nm..&out_table_nm_pool.(drop = model_master_rk);
  length MODEL_RK  SCORE_TIME_SK LEVEL_SK VARIABLE_SK
         SEQ_NO 8
		 VARIABLE_NAME $200 ATTRIBUTE_NAME $200 
		 SCORECARD_POINTS
		 NO_OF_RECORDS_DEV NO_OF_DEFAULTS_DEV AVG_SCORE_POINTS
         NO_OF_RECORDS_ACTUAL NO_OF_DEFAULTS_ACTUAL 8;
  set &debug_lib..&out_table_nm_pool.(rename  = (attr_name = attribute_name
                                                 var_name  = variable_name));

  if score_time_sk = . then score_time_sk =  &time_key;
  if model_rk  = . then model_rk = &model_key;
  if level_sk = . then level_sk  = &levelsk;
  if no_of_records_actual =. then no_of_records_actual = 0;
 %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	if no_of_records_actual_all =. then no_of_records_actual_all = 0;
  %end;
  if no_of_records_dev = . then no_of_records_dev =0;
  if no_of_defaults_actual  = . then no_of_defaults_actual =0;
  if no_of_defaults_dev =. then no_of_defaults_dev = 0;

run;
%bankfdn_rcset(&syserr);

proc sql;
  create unique index xpk_attr_feed_mip on 
    &out_lib_nm..&out_table_nm_pool.(model_rk,score_time_sk,variable_sk,seq_no);
quit;
%bankfdn_rcset(&sqlrc);

*==============================================================;
* Creating Score Range level feed ;
*==============================================================;

/* Actual Sample - Account level*/
proc sql;
  create table &debug_lib..&out_table_nm_acct._sql as
  select 
     score_card_variable_dim.variable_name as var_name length = 200,
	 score_card_dim.variable_group_label as attr_name length = 200, 
	 score_card_dim.score_card_sk,
	 score_card_dim.variable_sk,
	 score_card_dim.variable_group_no ,
	 _&mm_ext_key._sc_range_fact.from_score, 
	 _&mm_ext_key._sc_range_fact.to_score, 
	 _&model_key._&time_key._pd_fact.model_rk,
     _&model_key._&time_key._pd_fact.record_rk as record_rk,
	 _&model_key._&time_key._pd_fact.predicted_good_bad as predicted_good_bad,
     _&model_key._&time_key._pd_fact.actual_good_bad as actual_good_bad

  from &out_lib_nm_fact.._&model_key._&time_key._sc_fact
                  inner join 
       &out_lib_nm_dim..score_card_dim
  on ( _&model_key._&time_key._sc_fact.score_card_sk = score_card_dim.score_card_sk) 
                  inner join
       &out_lib_nm_dim..score_card_variable_dim
  on (score_card_dim.variable_sk = score_card_variable_dim.variable_sk)
                  right join
       &out_lib_nm_fact.._&mm_ext_key._sc_range_fact
  on (_&model_key._&time_key._sc_fact.score_card_sk = _&mm_ext_key._sc_range_fact.score_card_sk)
                  inner join
       &out_lib_nm_fact.._&model_key._&time_key._pd_fact
  on (
      _&model_key._&time_key._sc_fact.score_time_sk =_&model_key._&time_key._pd_fact.score_time_sk and 
	  _&model_key._&time_key._sc_fact.record_rk = _&model_key._&time_key._pd_fact.record_rk)

  where 
     _&model_key._&time_key._pd_fact.score_points > from_score and 
	 _&model_key._&time_key._pd_fact.score_points <= to_score;

quit;
%bankfdn_rcset(&sqlrc);
 
/* Actual sample - Group by to get the no of accounts */
proc sql;
  create table &debug_lib..&out_table_nm_acct._acp as
  select 
      model_rk, 
      variable_sk,
      var_name, 
      attr_name,
      from_score, 
      to_score, 
	  count(record_rk) as no_of_records_actual, 
      sum(actual_good_bad) as no_of_defaults_actual,
	  max(variable_group_no) as variable_group_no
  from &debug_lib..&out_table_nm_acct._sql a
  /* Subsetting the records for Applications that are ACCEPTED */
   %if &model_type=%str(&PD_APP_MODEL) %then %do;
	   inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
	   on (a.record_rk = b.application_rk)
   %end;
  group by model_rk,variable_sk,var_name,attr_name, from_score, to_score;
quit;
%bankfdn_rcset(&sqlrc);
%if &model_type=%str(&PD_APP_MODEL) %then %do;
	proc sql;
	  create table &debug_lib..&out_table_nm_acct._all as
	  select 
	      model_rk, 
	      variable_sk,
	      var_name, 
	      attr_name,
	      from_score, 
	      to_score, 
		  count(record_rk) as no_of_records_actual_all,
	  max(variable_group_no) as variable_group_no
	  from &debug_lib..&out_table_nm_acct._sql
	  group by model_rk,variable_sk,var_name,attr_name, from_score, to_score;
	quit;
	%bankfdn_rcset(&sqlrc);
%end;

%if &model_type=%str(&PD_APP_MODEL) %then %do;
	%let alias = b;
%end;
%else %do;
	%let alias = a;
%end;


proc sql;
  create table &debug_lib..&out_table_nm_acct._act as
   	select &alias..model_rk, 
	      &alias..variable_sk,
	      &alias..var_name, 
	      &alias..attr_name,
	      &alias..from_score, 
	      &alias..to_score, 
		  a.no_of_records_actual, 
	          a.no_of_defaults_actual,
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		 b.no_of_records_actual_all,
	%end;
		&alias..variable_group_no
	from &debug_lib..&out_table_nm_acct._acp a
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			right join &debug_lib..&out_table_nm_acct._all b
				on a.model_rk = b.model_rk and 
					a.variable_sk = b.variable_sk and
					a.var_name = b.var_name and
					a.attr_name = b.attr_name and
					a.from_score = b.from_score and
					a.to_score = b.to_score
		%end;
	;
quit;

/* DEV sample  */

proc sql;
  create table &debug_lib..&out_table_nm_acct._var as
  select 
    score_card_dim.variable_sk, 
    score_card_variable_dim.variable_name as var_name length = 200,
	score_card_dim.variable_group_label as attr_name length = 200,
	score_card_dim.variable_group_no,
	_&mm_ext_key._sc_range_fact.from_score, 
	_&mm_ext_key._sc_range_fact.to_score, 
	_&mm_ext_key._sc_range_fact.no_of_records, 
	_&mm_ext_key._sc_range_fact.no_of_defaults 
  from  &out_lib_nm_dim..score_card_dim 
                inner join
        &out_lib_nm_dim..score_card_variable_dim
  on (score_card_dim.variable_sk = score_card_variable_dim.variable_sk)
                inner join
        &out_lib_nm_fact.._&mm_ext_key._sc_range_fact
  on (score_card_dim.score_card_sk = _&mm_ext_key._sc_range_fact.score_card_sk)
  where  score_card_dim.model_rk = &mm_ext_key;
quit;
%bankfdn_rcset(&sqlrc);
 
/* Merging DEV sample & Actual Sample  */
proc sql;
  create table &debug_lib..&out_table_nm_acct. as
  select &out_table_nm_acct._act.model_rk,
    &out_table_nm_acct._var.variable_sk,
	&out_table_nm_acct._var.var_name,
    &out_table_nm_acct._var.attr_name,
	&out_table_nm_acct._var.from_score, 
	&out_table_nm_acct._var.to_score, 

	"> "||ktrim(kleft(put(&out_table_nm_acct._var.from_score,8.)))||" TO <= "||ktrim(kleft(put(&out_table_nm_acct._var.to_score,8.)))
     as score_range length = 50,
 
	no_of_records_actual, 
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		no_of_records_actual_all, 
	%end;
	no_of_defaults_actual, 
	&out_table_nm_acct._var.no_of_records as no_of_records_dev, 
	&out_table_nm_acct._var.no_of_defaults as no_of_defaults_dev, 
	&out_table_nm_acct._var.variable_group_no
	
  from &debug_lib..&out_table_nm_acct._var 
	              left join
       &debug_lib..&out_table_nm_acct._act
  on (&out_table_nm_acct._act.variable_sk = &out_table_nm_acct._var.variable_sk and
	  &out_table_nm_acct._act.variable_group_no = &out_table_nm_acct._var.variable_group_no and
	  &out_table_nm_acct._act.from_score = &out_table_nm_acct._var.from_score and
	  &out_table_nm_acct._act.to_score = &out_table_nm_acct._var.to_score);
quit;
%bankfdn_rcset(&sqlrc);

proc sql noprint;
  select max(level_sk) into
    :levelsk
  from &debug_lib..&out_table_nm_pool.;
quit;
%bankfdn_rcset(&sqlrc);

data  &out_lib_nm..&out_table_nm_acct.(drop = variable_group_no);
  length MODEL_RK  SCORE_TIME_SK LEVEL_SK VARIABLE_SK
         SEQ_NO 8
		 VARIABLE_NAME $200 ATTRIBUTE_NAME $200 SCORE_RANGE $50
         TO_SCORE FROM_SCORE 
		 NO_OF_RECORDS_DEV NO_OF_DEFAULTS_DEV
         NO_OF_RECORDS_ACTUAL NO_OF_DEFAULTS_ACTUAL 8;
  set   &debug_lib..&out_table_nm_acct.(rename  = (attr_name = attribute_name
                                                    var_name  = variable_name));

  if score_time_sk = . then score_time_sk =  &time_key;
  if model_rk  = . then model_rk = &model_key;
  
  if level_sk = . then level_sk  = &levelsk;
  if no_of_records_actual =. then no_of_records_actual = 0;
  %if &model_type=%str(&PD_APP_MODEL) %then %do;
  	if no_of_records_actual_all =. then no_of_records_actual_all = 0;
  %end;
  if no_of_records_dev = . then no_of_records_dev =0;
  if no_of_defaults_actual  = . then no_of_defaults_actual =0;
  if no_of_defaults_dev =. then no_of_defaults_dev = 0;

  seq_no = variable_group_no;

run;
%bankfdn_rcset(&syserr);

proc sql;
  create unique index xpk_scr_range_feed_mip on 
    &out_lib_nm..&out_table_nm_acct.(model_rk,score_time_sk,variable_sk,seq_no,score_range);
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_feed_pd_mip;
