/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_update_lgd_actuals.sas
Function       : This macro updates individual _<MODEL_SK>_<SCORE_TIME_SK>_LGD_FACT
                 from taking the values from LGD_ACTUALS table.

Authors        :  CSB Team
Date           :  17AUG2010
SAS            :  9.2
Called-by      :  UPDATE_LGD_FACT job
Calls          :  %RCSET macro to capture the return code for error handling.

Input Datasets :  LGD_ACTUALS (CSB Scratch library)
                  _<MODEL_SK>_<SCORE_TIME_SK>_LGD_FACT (CSB Fact library)

Output Datasets:  _<MODEL_SK>_<SCORE_TIME_SK>_LGD_FACT (CSB Fact library)

Logic          :  1. Take the distinct combination of MODEL_SK & SCORE_TIME_SK
                     in a dataset.
                  2. For each combination of MODEL_SK & SCORE_TIME_SK, update 
                     the ACTUALS in corresponding fact table.

Parameters     :  INDATA  -> name of input dataset containing the ACTUALS value
                             (with libref)
                  FACTLIB -> name of the library where LGD fact tables are present
**********************************************************************************************/

%macro csbmva_update_lgd_actuals(indata = ,factlib = );

%if %sysfunc(exist(&indata)) eq 0 %then %do;
  %return;
%end;

proc sql;
  create table csbscr.lgd_updatetime_list as
  	select distinct model_rk, score_time_sk
	from &indata;
quit;

%let dsid=%sysfunc(open(csbscr.lgd_updatetime_list));
%let totalobs=%sysfunc(attrn(&dsid,nobs));
%let rc=%sysfunc(close(&dsid));

%do i = 1 %to &totalobs;

data _null_;
  pointer = &i;
  set csbscr.lgd_updatetime_list point = pointer;

  call symput('m_model_key',model_rk);
  call symput('m_time_key',score_time_sk);
  stop;
run;

%let m_model_key = &m_model_key;
%let m_time_key = &m_time_key;

data &factlib.._&m_model_key._&m_time_key._lgd_fact;
  set &indata(where = (model_rk = &m_model_key and score_time_sk = &m_time_key)
              rename = (actual_loss = t_actual_loss
                        actual_lgd = t_actual_lgd
						exposure_at_default = t_ead
                        default_time_sk = t_default_time_sk)
			  idxname = idx_model_scoretime
             );
  modify &factlib.._&m_model_key._&m_time_key._lgd_fact key = xpk_lgd_fact / unique;

  actual_loss = t_actual_loss;
  actual_lgd = t_actual_lgd;
  exposure_at_default = t_ead;
  default_time_sk = t_default_time_sk;
  recovery_end_time_sk = default_time_sk + target_time_sk - score_time_sk;

  if _iorc_ = %sysrc(_SOK) then replace;

  _iorc_ = 0;
  _error_ = 0;

run;

%bankfdn_rcset(&syserr);

%end;

Proc append base=csbscr.lgd_updatetime_list data=csbscr.lgd_model_scoretime_list(drop=model_type_cd);
run;

%bankfdn_rcset(&syserr);

proc sort data=csbscr.lgd_updatetime_list nodup;
by _all_;
run;

%bankfdn_rcset(&syserr);

%mend csbmva_update_lgd_actuals;
